@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevsvg.c 9038 2008-08-28 02:40:07Z giles $ */
+/* $Id: gdevsvg.c 10067 2009-09-10 17:41:10Z giles $ */
 /* SVG (Scalable Vector Graphics) output device */
 
 #include "string_.h"
@@ -118,12 +118,13 @@ static dev_proc_put_params(svg_put_params);
         NULL			/* strip_copy_rop */\
 }
 
-gs_public_st_suffix_add0_final(st_device_svg, gx_device_svg,
+gs_public_st_suffix_string2_final(st_device_svg, gx_device_svg,
                                "gx_device_svg",
                                device_svg_enum_ptrs,
 			       device_svg_reloc_ptrs,
                                gx_device_finalize,
-			       st_device_vector);
+			       st_device_vector,
+			       strokecolor, fillcolor);
 
 /* The output device is named 'svg' but we're referred to as the
    'svgwrite' device by the build system to avoid conflicts with
