@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsdevice.c 8602 2008-03-19 10:09:21Z ken $ */
+/* $Id: gsdevice.c 10292 2009-11-07 02:31:37Z ray $ */
 /* Device operators for Ghostscript library */
 #include "ctype_.h"
 #include "memory_.h"		/* for memchr, memcpy */
@@ -129,6 +129,16 @@ gs_output_page(gs_state * pgs, int num_copies, int flush)
 {
     gx_device *dev = gs_currentdevice(pgs);
 
+    /* for devices that hook 'fill_path' in order to pick up imager state */
+    /* values such as dev_ht (such as tiffsep1), make a dummy call here   */
+    /* to make sure that it has been called at least once		  */
+    gs_gsave(pgs);
+    gs_newpath(pgs);
+    gs_moveto(pgs, 0.0, 0.0);
+    gs_setgray(pgs, 0.0);
+    gs_fill(pgs);
+    gs_grestore(pgs);
+
     if (dev->IgnoreNumCopies)
 	num_copies = 1;
     return (*dev_proc(dev, output_page)) (dev, num_copies, flush);
