@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxclimag.c 9285 2008-12-13 11:14:54Z leonardo $ */
+/*$Id: gxclimag.c 10200 2009-10-21 03:50:46Z mvrhel $ */
 /* Higher-level image operations for band lists */
 #include "math_.h"
 #include "memory_.h"
@@ -475,7 +475,7 @@ clist_begin_typed_image(gx_device * dev,
 	 * worthwhile.
 	 */
 	gx_color_index all =
-	    ((gx_color_index)1 << dev->color_info.depth) - 1;
+	    ((gx_color_index)1 << cdev->clist_color_info.depth) - 1;
 
 	if (bits_per_pixel > 4 || pim->Interpolate || num_components > 1)
 	    colors_used = all;
@@ -536,13 +536,22 @@ clist_begin_typed_image(gx_device * dev,
     return 0;
 
     /*
-     * We couldn't handle the image.  Use the default algorithms, which
+     * We couldn't handle the image.  It is up to the caller to
+     * use the default algorithms, which
      * break the image up into rectangles or small pixmaps.
+     * If we are doing the PDF14 transparency device
+     * then we want to make sure we do NOT use the target 
+     * device.  In this case we return -1.
      */
 use_default:
     gs_free_object(mem, pie, "clist_begin_typed_image");
-    return gx_default_begin_typed_image(dev, pis, pmat, pic, prect,
-					pdcolor, pcpath, mem, pinfo);
+
+    if (pis->has_transparency){
+        return -1;
+    } else {
+        return gx_default_begin_typed_image(dev, pis, pmat, pic, prect,
+					    pdcolor, pcpath, mem, pinfo);
+    }
 }
 
 /* Error cleanup for clist_image_plane_data. */
@@ -878,10 +887,13 @@ clist_create_compositor(gx_device * dev,
     if (code < 0)
         return code;
 
-    code = pcte->type->procs.get_cropping(pcte, &ry, &rheight);
+    code = pcte->type->procs.get_cropping(pcte, &ry, &rheight, cdev->cropping_min, cdev->cropping_max);
+
     if (code < 0)
 	return code;
+
     cropping_op = code;
+
     if (cropping_op == 1) {
 	first_band = ry / band_height;
 	last_band = (ry + rheight + band_height - 1) / band_height;
@@ -889,10 +901,33 @@ clist_create_compositor(gx_device * dev,
 	first_band = cdev->cropping_min / band_height;
 	last_band = (cdev->cropping_max + band_height - 1) / band_height;
     }
+
     if (last_band - first_band > no_of_bands * 2 / 3) {
 	/* Covering many bands, so write "all bands" command for shorter clist. */
 	cropping_op = 0;
     }
+
+    /* Using 'v' here instead of 'L' since this is used almost exclusively with
+       the transparency code */
+ 
+    if (gs_debug_c('v')) {
+
+        if(cropping_op != 0) {
+
+           dprintf2("[v] cropping_op = %d. Total number of bands is %d \n",
+		     cropping_op, no_of_bands);
+           dprintf2("[v]  Writing out from band %d through band %d \n",
+		     first_band, last_band);
+
+        } else {
+
+           dprintf1("[v] cropping_op = %d. Writing out to all bands \n",
+		     cropping_op);
+
+        }
+
+    }
+   
     if (cropping_op == 0) {
 	/* overprint applies to all bands */
 	size_dummy = size;
@@ -1488,7 +1523,7 @@ cmd_image_plane_data(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 	code = cmd_put_set_data_x(cldev, pcls, data_x);
 	if (code < 0)
 	    return code;
-	offset = ((data_x & ~7) * cldev->color_info.depth) >> 3;
+	offset = ((data_x & ~7) * cldev->clist_color_info.depth) >> 3;
     }
     code = set_cmd_put_op(dp, cldev, pcls, cmd_opv_image_data, len);
     if (code < 0)
