@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxclist.c 9288 2008-12-13 20:05:37Z leonardo $ */
+/*$Id: gxclist.c 10200 2009-10-21 03:50:46Z mvrhel $ */
 /* Command list document- and page-level code. */
 #include "memory_.h"
 #include "string_.h"
@@ -164,7 +164,10 @@ const gx_device_procs gs_clist_device_procs = {
     clist_fill_linear_color_triangle,
     gx_forward_update_spot_equivalent_colors,
     gx_forward_ret_devn_params,
-    clist_fillpage
+    clist_fillpage,
+    NULL,                      /* push_transparency_state */
+    NULL                       /* pop_transparency_state */
+
 };
 
 /*------------------- Choose the implementation -----------------------
@@ -190,7 +193,7 @@ clist_init_io_procs(gx_device_clist *pclist_dev, bool in_memory)
 /* Initialization for imager state. */
 /* The initial scale is arbitrary. */
 const gs_imager_state clist_imager_state_initial =
-{gs_imager_state_initial(300.0 / 72.0)};
+{gs_imager_state_initial(300.0 / 72.0, false)};
 
 /*
  * The buffer area (data, data_size) holds a bitmap cache when both writing
@@ -358,6 +361,16 @@ clist_init_data(gx_device * dev, byte * init_data, uint data_size)
     gx_device *pbdev = (gx_device *)&bdev;
     int code;
 
+    /* the clist writer has its own color info that depends upon the 
+       transparency group color space (if transparency exists).  The data that is
+       used in the clist writing. Here it is initialized with 
+       the target device color info.  The values will be pushed and popped
+       in a stack if we have changing color spaces in the transparency groups. */
+
+    cdev->clist_color_info.depth = dev->color_info.depth;
+    cdev->clist_color_info.polarity = dev->color_info.polarity;
+    cdev->clist_color_info.num_components = dev->color_info.num_components;
+    
     /* Call create_buf_device to get the memory planarity set up. */
     cdev->buf_procs.create_buf_device(&pbdev, target, 0, NULL, NULL, clist_get_band_complexity(0, 0));
     /* HACK - if the buffer device can't do copy_alpha, disallow */
