@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxclist.h 9288 2008-12-13 20:05:37Z leonardo $ */
+/* $Id: gxclist.h 9666 2009-04-20 19:16:24Z mvrhel $ */
 /* Command list definitions for Ghostscript. */
 /* Requires gxdevice.h and gxdevmem.h */
 
@@ -281,6 +281,14 @@ struct gx_device_clist_writer_s {
     uint mask_id;
     uint temp_mask_id; /* Mask id of a mask of an image with SMask. */
     bool is_fillpage;
+    gx_device_color_info clist_color_info; /* color information to be used during clist writing.  
+                                           It may be different than the target device if we 
+                                           are in a transparency group.  Since the fill rect 
+                                           commands use the forward procs and we have no 
+                                           access to the graphic state information in those
+                                           routines, this is the logical place to put this
+                                           information */
+
 };
 #ifndef gx_device_clist_writer_DEFINED
 #define gx_device_clist_writer_DEFINED
