@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxclrast.c 9852 2009-07-09 05:59:44Z henrys $ */
+/*$Id: gxclrast.c 10624 2010-01-20 18:11:39Z mvrhel $ */
 /* Command list interpreter/rasterizer */
 #include "memory_.h"
 #include "gx.h"
@@ -127,7 +127,7 @@ typedef struct command_buf_s {
     byte *data;			/* actual buffer, guaranteed aligned */
     uint size;
     const byte *ptr;		/* next byte to be read (see above) */
-    const byte *limit;		/* refill warning point */
+    const byte *warn_limit;	/* refill warning point */
     const byte *end;		/* byte just beyond valid data */
     stream *s;			/* for refilling buffer */
     int end_status;
@@ -138,9 +138,11 @@ static void
 set_cb_end(command_buf_t *pcb, const byte *end)
 {
     pcb->end = end;
-    pcb->limit = pcb->data + (pcb->size - cmd_largest_size + 1);
-    if ( pcb->limit > pcb->end )
-	pcb->limit = pcb->end;
+    pcb->warn_limit = pcb->data + (pcb->size - cmd_largest_size + 1);
+    if ( pcb->warn_limit > pcb->end )
+	pcb->warn_limit = pcb->end;	/**** This is dangerous. Other places ****/
+					/**** assume that the limit is a soft ****/
+					/**** limit and should check 'end'    ****/
 }
 
 /* Read more data into a command buffer. */
@@ -621,7 +623,7 @@ in:				/* Initialize for a new page. */
 	gs_logical_operation_t log_op;
 
 	/* Make sure the buffer contains a full command. */
-	if (cbp >= cbuf.limit) {
+	if (cbp >= cbuf.warn_limit) {
 	    if (cbuf.end_status < 0) {	/* End of file or error. */
 		if (cbp >= cbuf.end) {
 		    code = (cbuf.end_status == EOFC ? 0 :
@@ -1457,7 +1459,7 @@ idata:			data_size = 0;
 				       for reducing time and memory expense. */
 				    int len;
 
-				    if (cbp >= cbuf.limit) {
+				    if (cbp >= cbuf.warn_limit) {
 					code = top_up_cbuf(&cbuf, &cbp);
 					if (code < 0)
 					    goto out;
@@ -1650,7 +1652,7 @@ idata:			data_size = 0;
 					    goto out;
 				    }
 				    while (left) {
-					if (cbp + left > cbuf.limit) {
+					if (cbuf.warn_limit - cbp < (int)left) {  /* cbp can be past warn_limit */
 					    code = top_up_cbuf(&cbuf, &cbp);
 					    if (code < 0)
 						return code;
@@ -1783,6 +1785,7 @@ idata:			data_size = 0;
 			    break;
 			case cmd_opv_stroke:
 			    stroke_params.flatness = imager_state.flatness;
+			    stroke_params.traditional = false;
 			    code = gx_stroke_path_only(ppath,
 						       (gx_path *) 0, tdev,
 					      &imager_state, &stroke_params,
@@ -1806,7 +1809,20 @@ idata:			data_size = 0;
 				    clipper_dev_open = true;
 				}
 				if (clipper_dev_open)
-				    ttdev = (gx_device *)&clipper_dev;
+				    ttdev = (gx_device *)&clipper_dev;                                
+                                /* Note that if we have transparency present, the clipper device may need to have
+                                   its color information updated to be synced up with the target device.
+                                   This can occur if we had fills of a path first with a transparency mask to get
+                                   an XPS opacity followed by a fill with a transparency group. This occurs in
+                                   the XPS gradient code */
+                                if (tdev->color_info.num_components != ttdev->color_info.num_components){
+                                    /* Reset the clipper device color information. Only worry about
+                                       the information that is used in the trap code */
+                                    ttdev->color_info.num_components = tdev->color_info.num_components;
+                                    ttdev->color_info.depth = tdev->color_info.depth;
+                                    memcpy(&(ttdev->color_info.comp_bits),&(tdev->color_info.comp_bits),GX_DEVICE_COLOR_MAX_COMPONENTS);
+                                    memcpy(&(ttdev->color_info.comp_shift),&(tdev->color_info.comp_shift),GX_DEVICE_COLOR_MAX_COMPONENTS);
+                                }
 				cmd_getw(left.start.x, cbp);
 				cmd_getw(left.start.y, cbp);
 				cmd_getw(left.end.x, cbp);
@@ -2315,10 +2331,14 @@ read_ht_segment(
 
     /* get the segment size; refill command buffer if necessary */
     enc_u_getw(seg_size, cbp);
-    if (cbp + seg_size > pcb->limit) {
+    if (pcb->warn_limit - cbp < (int)seg_size) { /* cbp can be past warn_limit */
         code = top_up_cbuf(pcb, &cbp);
 	if (code < 0)
 	    return code;
+	if (pcb->end - cbp < (int)seg_size) {
+	    eprintf(" *** ht segment size doesn't fit in buffer ***\n");
+            return_error(gs_error_unknownerror);
+	}
     }
 
     if (pht_buff->pbuff == 0) {
@@ -2365,10 +2385,12 @@ read_set_misc2(command_buf_t *pcb, gs_imager_state *pis, segment_notes *pnotes)
     cmd_getw(mask, cbp);
     if (mask & cap_join_known) {
 	cb = *cbp++;
-	pis->line_params.cap = (gs_line_cap)((cb >> 3) & 7);
+        pis->line_params.start_cap =
+            pis->line_params.end_cap =
+                pis->line_params.dash_cap = (gs_line_cap)((cb >> 3) & 7);
 	pis->line_params.join = (gs_line_join)(cb & 7);
 	if_debug2('L', " cap=%d join=%d\n",
-		  pis->line_params.cap, pis->line_params.join);
+		  pis->line_params.start_cap, pis->line_params.join);
     }
     if (mask & cj_ac_sa_known) {
 	cb = *cbp++;
@@ -2653,7 +2675,7 @@ read_create_compositor(
     
     /* If we read more than the maximum expected, return a rangecheck error */
     if ( code > MAX_CLIST_COMPOSITOR_SIZE )
- 		return_error(gs_error_rangecheck);
+	return_error(gs_error_rangecheck);
    	
     if (code > 0)
         cbp += code;
