@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: stream.c 9311 2009-01-01 05:34:09Z alexcher $ */
+/* $Id: stream.c 10257 2009-11-02 17:36:56Z alexcher $ */
 /* Stream package for Ghostscript interpreter */
 #include "stdio_.h"		/* includes std.h */
 #include "memory_.h"
@@ -132,6 +132,7 @@ s_init_state(stream_state *st, const stream_template *template,
     st->memory = mem;
     st->report_error = s_no_report_error;
     st->min_left = 0;
+    st->error_string[0] = 0;
 }
 stream_state *
 s_alloc_state(gs_memory_t * mem, gs_memory_type_ptr_t stype,
@@ -1259,9 +1260,9 @@ s_close_disable(stream *s)
 int
 file_close_disable(stream * s)
 {
-    int code = (*s->save_close)(s);
-
-    if (code)
+    int code;
+    
+    if ((*s->save_close != NULL) && ((code = (*s->save_close)(s)) != 0))
 	return code;
     s_close_disable(s);
     return file_close_finish(s);
