@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: imainarg.c 9043 2008-08-28 22:48:19Z giles $ */
+/* $Id: imainarg.c 10463 2009-12-08 06:31:51Z alexcher $ */
 /* Command line parsing and dispatching */
 #include "ctype_.h"
 #include "memory_.h"
@@ -202,6 +202,14 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
 		    return code;
 		if (code > 0)
 		    outprintf(minst->heap, "Unknown switch %s - ignoring\n", arg);
+		if (gs_debug[':'] && arg[1] == 'Z') {
+		    int i;
+
+		    dprintf1("%% Init started, instance 0x%p, with args: ", minst);
+		    for (i=1; i<argc; i++)
+			dprintf1("%s ", argv[i]);
+		    dprintf("\n");
+		}
 		break;
 	    default:
 		code = argproc(minst, arg);
@@ -212,11 +220,18 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
     if (code < 0)
 	return code;
 
-
     code = gs_main_init2(minst);
     if (code < 0)
 	return code;
 
+    if (gs_debug[':']) {
+	int i;
+
+	dprintf1("%% Init done, instance 0x%p, with args: ", minst);
+	for (i=1; i<argc; i++)
+	    dprintf1("%s ", argv[i]);
+	dprintf("\n");
+    }
     if (!minst->run_start)
 	return e_Quit;
     return code ;
@@ -736,8 +751,10 @@ run_buffered(gs_main_instance * minst, const char *arg)
 	return_error(e_invalidfileaccess);
     }
     code = gs_main_init2(minst);
-    if (code < 0)
+    if (code < 0) {
+        fclose(in);
 	return code;
+    }
     code = gs_main_run_string_begin(minst, minst->user_errors,
 				    &exit_code, &error_object);
     if (!code) {
@@ -962,7 +979,7 @@ print_devices(const gs_main_instance *minst)
 	const char **names;
 	size_t ndev = 0;
 
-	for (i = 0; (pdev = gs_getdevice(i)) != 0; i++)
+	for (i = 0; gs_getdevice(i) != 0; i++)
 	    ;
 	ndev = (size_t)i;
 	names = (const char **)gs_alloc_bytes(minst->heap, ndev * sizeof(const char*), "print_devices");
