@@ -12,6 +12,8 @@
 # or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
 # San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 
+# $Id: check_dirs.py 10701 2010-02-03 22:39:42Z giles $
+
 # Check the contents of Ghostscript directories.
 
 import os
@@ -40,7 +42,8 @@ class GSCheckEmptyTopDirectory(GSTestCase):
 ################ Main program
 
 gsFilesInTopDirectory = ['LICENSE', 'autogen.sh', 'Makefile',
-			 'configure', 'config.log', 'config.status']
+			 'configure', 'config.log', 'config.status',
+			 'ghostscript.vcproj']
 # Add the tests defined in this file to a suite.
 
 def addTests(suite, gsroot, **args):
